# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class PaymentStatus(str, Enum):
    """
    Payment status
    """

    """
    allowed enum values
    """
    WAITING = "waiting"
    PROCESSING = "processing"
    PAID = "paid"
    PARTIALLY_PAID = "partially_paid"
    REFUNDED = "refunded"
    FAILED = "failed"
    CANCELLED = "cancelled"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of PaymentStatus from a JSON string"""
        return cls(json.loads(json_str))
