# coding: utf-8

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
)
from typing_extensions import Self


class WalletNonceRequest(BaseModel):
    """
    WalletNonceRequest
    """  # noqa: E501

    host: StrictStr
    origin: StrictStr
    address: StrictStr
    chain_id: Optional[Union[StrictFloat, StrictInt]] = Field(
        default=56, alias="chainId"
    )
    statement: Optional[StrictStr] = "Sign in with Binance Smart Chain wallet"
    raise_if_new: Optional[StrictBool] = Field(
        default=True,
        description="If true, will raise an error if the wallet is not found",
        alias="raiseIfNew",
    )
    __properties: ClassVar[List[str]] = [
        "host",
        "origin",
        "address",
        "chainId",
        "statement",
        "raiseIfNew",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WalletNonceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WalletNonceRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "host": obj.get("host"),
                "origin": obj.get("origin"),
                "address": obj.get("address"),
                "chainId": obj.get("chainId") if obj.get("chainId") is not None else 56,
                "statement": (
                    obj.get("statement")
                    if obj.get("statement") is not None
                    else "Sign in with Binance Smart Chain wallet"
                ),
                "raiseIfNew": (
                    obj.get("raiseIfNew") if obj.get("raiseIfNew") is not None else True
                ),
            }
        )
        return _obj
