# coding: utf-8

# flake8: noqa

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# Define package exports
__all__ = [
    "AdminApi",
    "AuthApi",
    "UserApi",
    "WalletApi",
    "ApiResponse",
    "ApiClient",
    "Configuration",
    "OpenApiException",
    "ApiTypeError",
    "ApiValueError",
    "ApiKeyError",
    "ApiAttributeError",
    "ApiException",
    "AddWallet200Response",
    "AddWalletRequest",
    "AuthorizeUser200Response",
    "AuthorizeUserRequest",
    "CreateApiKey200Response",
    "CreateApiKeyRequest",
    "CreateUser200Response",
    "CreateUser200ResponseAuth",
    "CreateUser200ResponseAuthAuth",
    "CreateUserRequest",
    "CreateUserWithWalletRequest",
    "ErrorBADREQUEST",
    "ErrorFORBIDDEN",
    "ErrorINTERNALSERVERERROR",
    "ErrorNOTFOUND",
    "ErrorUNAUTHORIZED",
    "GenerateOtpCodeRequest",
    "GetApiKeys200ResponseInner",
    "ListWallets200Response",
    "ListWallets200ResponseDataInner",
    "RefreshTokenInfo200Response",
    "RefreshTokenInfo200ResponseUserSession",
    "ResendVerificationEmailRequest",
    "RevokeAllSessions200Response",
    "RevokeUserTokens200Response",
    "RevokeUserTokensRequest",
    "TokenInfo200Response",
    "UnlinkWalletRequest",
    "UpdateUserRequest",
    "UpdateWalletRequest",
    "UserByUsername200Response",
    "UserResetPasswordRequest",
    "UserSetPasswordRequest",
    "VerifyEmail200Response",
    "VerifyEmail200ResponseAuth",
    "VerifyEmailRequest",
    "VerifyOtpCode200Response",
    "VerifyOtpCodeRequest",
    "VerifyWalletRequest",
    "WalletNonceRequest",
    "Whoami200Response",
]

# import apis into sdk package
from crypticorn.auth.client.api.admin_api import AdminApi as AdminApi
from crypticorn.auth.client.api.auth_api import AuthApi as AuthApi
from crypticorn.auth.client.api.user_api import UserApi as UserApi
from crypticorn.auth.client.api.wallet_api import WalletApi as WalletApi
from crypticorn.auth.client.api_client import ApiClient as ApiClient

# import ApiClient
from crypticorn.auth.client.api_response import ApiResponse as ApiResponse
from crypticorn.auth.client.configuration import Configuration as Configuration
from crypticorn.auth.client.exceptions import ApiAttributeError as ApiAttributeError
from crypticorn.auth.client.exceptions import ApiException as ApiException
from crypticorn.auth.client.exceptions import ApiKeyError as ApiKeyError
from crypticorn.auth.client.exceptions import ApiTypeError as ApiTypeError
from crypticorn.auth.client.exceptions import ApiValueError as ApiValueError
from crypticorn.auth.client.exceptions import OpenApiException as OpenApiException

# import models into sdk package
from crypticorn.auth.client.models.add_wallet200_response import (
    AddWallet200Response as AddWallet200Response,
)
from crypticorn.auth.client.models.add_wallet_request import (
    AddWalletRequest as AddWalletRequest,
)
from crypticorn.auth.client.models.authorize_user200_response import (
    AuthorizeUser200Response as AuthorizeUser200Response,
)
from crypticorn.auth.client.models.authorize_user_request import (
    AuthorizeUserRequest as AuthorizeUserRequest,
)
from crypticorn.auth.client.models.create_api_key200_response import (
    CreateApiKey200Response as CreateApiKey200Response,
)
from crypticorn.auth.client.models.create_api_key_request import (
    CreateApiKeyRequest as CreateApiKeyRequest,
)
from crypticorn.auth.client.models.create_user200_response import (
    CreateUser200Response as CreateUser200Response,
)
from crypticorn.auth.client.models.create_user200_response_auth import (
    CreateUser200ResponseAuth as CreateUser200ResponseAuth,
)
from crypticorn.auth.client.models.create_user200_response_auth_auth import (
    CreateUser200ResponseAuthAuth as CreateUser200ResponseAuthAuth,
)
from crypticorn.auth.client.models.create_user_request import (
    CreateUserRequest as CreateUserRequest,
)
from crypticorn.auth.client.models.create_user_with_wallet_request import (
    CreateUserWithWalletRequest as CreateUserWithWalletRequest,
)
from crypticorn.auth.client.models.error_badrequest import (
    ErrorBADREQUEST as ErrorBADREQUEST,
)
from crypticorn.auth.client.models.error_forbidden import (
    ErrorFORBIDDEN as ErrorFORBIDDEN,
)
from crypticorn.auth.client.models.error_internalservererror import (
    ErrorINTERNALSERVERERROR as ErrorINTERNALSERVERERROR,
)
from crypticorn.auth.client.models.error_notfound import ErrorNOTFOUND as ErrorNOTFOUND
from crypticorn.auth.client.models.error_unauthorized import (
    ErrorUNAUTHORIZED as ErrorUNAUTHORIZED,
)
from crypticorn.auth.client.models.generate_otp_code_request import (
    GenerateOtpCodeRequest as GenerateOtpCodeRequest,
)
from crypticorn.auth.client.models.get_api_keys200_response_inner import (
    GetApiKeys200ResponseInner as GetApiKeys200ResponseInner,
)
from crypticorn.auth.client.models.list_wallets200_response import (
    ListWallets200Response as ListWallets200Response,
)
from crypticorn.auth.client.models.list_wallets200_response_data_inner import (
    ListWallets200ResponseDataInner as ListWallets200ResponseDataInner,
)
from crypticorn.auth.client.models.refresh_token_info200_response import (
    RefreshTokenInfo200Response as RefreshTokenInfo200Response,
)
from crypticorn.auth.client.models.refresh_token_info200_response_user_session import (
    RefreshTokenInfo200ResponseUserSession as RefreshTokenInfo200ResponseUserSession,
)
from crypticorn.auth.client.models.resend_verification_email_request import (
    ResendVerificationEmailRequest as ResendVerificationEmailRequest,
)
from crypticorn.auth.client.models.revoke_all_sessions200_response import (
    RevokeAllSessions200Response as RevokeAllSessions200Response,
)
from crypticorn.auth.client.models.revoke_user_tokens200_response import (
    RevokeUserTokens200Response as RevokeUserTokens200Response,
)
from crypticorn.auth.client.models.revoke_user_tokens_request import (
    RevokeUserTokensRequest as RevokeUserTokensRequest,
)
from crypticorn.auth.client.models.token_info200_response import (
    TokenInfo200Response as TokenInfo200Response,
)
from crypticorn.auth.client.models.unlink_wallet_request import (
    UnlinkWalletRequest as UnlinkWalletRequest,
)
from crypticorn.auth.client.models.update_user_request import (
    UpdateUserRequest as UpdateUserRequest,
)
from crypticorn.auth.client.models.update_wallet_request import (
    UpdateWalletRequest as UpdateWalletRequest,
)
from crypticorn.auth.client.models.user_by_username200_response import (
    UserByUsername200Response as UserByUsername200Response,
)
from crypticorn.auth.client.models.user_reset_password_request import (
    UserResetPasswordRequest as UserResetPasswordRequest,
)
from crypticorn.auth.client.models.user_set_password_request import (
    UserSetPasswordRequest as UserSetPasswordRequest,
)
from crypticorn.auth.client.models.verify_email200_response import (
    VerifyEmail200Response as VerifyEmail200Response,
)
from crypticorn.auth.client.models.verify_email200_response_auth import (
    VerifyEmail200ResponseAuth as VerifyEmail200ResponseAuth,
)
from crypticorn.auth.client.models.verify_email_request import (
    VerifyEmailRequest as VerifyEmailRequest,
)
from crypticorn.auth.client.models.verify_otp_code200_response import (
    VerifyOtpCode200Response as VerifyOtpCode200Response,
)
from crypticorn.auth.client.models.verify_otp_code_request import (
    VerifyOtpCodeRequest as VerifyOtpCodeRequest,
)
from crypticorn.auth.client.models.verify_wallet_request import (
    VerifyWalletRequest as VerifyWalletRequest,
)
from crypticorn.auth.client.models.wallet_nonce_request import (
    WalletNonceRequest as WalletNonceRequest,
)
from crypticorn.auth.client.models.whoami200_response import (
    Whoami200Response as Whoami200Response,
)
