# coding: utf-8

# flake8: noqa
"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.trade.client.models.actions_count import ActionsCount
from crypticorn.trade.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.trade.client.models.api_error_level import ApiErrorLevel
from crypticorn.trade.client.models.api_error_type import ApiErrorType
from crypticorn.trade.client.models.bot import Bot
from crypticorn.trade.client.models.bot_create import BotCreate
from crypticorn.trade.client.models.bot_status import BotStatus
from crypticorn.trade.client.models.bot_update import BotUpdate
from crypticorn.trade.client.models.exception_detail import ExceptionDetail
from crypticorn.trade.client.models.exchange import Exchange
from crypticorn.trade.client.models.exchange_info import ExchangeInfo
from crypticorn.trade.client.models.exchange_key import ExchangeKey
from crypticorn.trade.client.models.exchange_key_balance import ExchangeKeyBalance
from crypticorn.trade.client.models.exchange_key_create import ExchangeKeyCreate
from crypticorn.trade.client.models.exchange_key_update import ExchangeKeyUpdate
from crypticorn.trade.client.models.execution_ids import ExecutionIds
from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.futures_trading_action import FuturesTradingAction
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.margin_mode import MarginMode
from crypticorn.trade.client.models.market_type import MarketType
from crypticorn.trade.client.models.notification import Notification
from crypticorn.trade.client.models.notification_create import NotificationCreate
from crypticorn.trade.client.models.notification_update import NotificationUpdate
from crypticorn.trade.client.models.order import Order
from crypticorn.trade.client.models.order_status import OrderStatus
from crypticorn.trade.client.models.orders_count import OrdersCount
from crypticorn.trade.client.models.paginated_response_futures_trading_action import (
    PaginatedResponseFuturesTradingAction,
)
from crypticorn.trade.client.models.paginated_response_order import (
    PaginatedResponseOrder,
)
from crypticorn.trade.client.models.pn_l import PnL
from crypticorn.trade.client.models.post_futures_action import PostFuturesAction
from crypticorn.trade.client.models.spot_balance import SpotBalance
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_exchange_info import StrategyExchangeInfo
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.models.tpsl import TPSL
from crypticorn.trade.client.models.tpsl_create import TPSLCreate
from crypticorn.trade.client.models.trading_action_type import TradingActionType
