# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.spot_balance import SpotBalance


class ExchangeKeyBalance(BaseModel):
    """
    Balance information for an exchange API key including futures and spot balances.
    """  # noqa: E501

    api_key_id: StrictStr = Field(description="API key ID.")
    futures: FuturesBalance = Field(description="Futures balance information.")
    spot: List[SpotBalance] = Field(description="Spot balance information.")
    __properties: ClassVar[List[str]] = ["api_key_id", "futures", "spot"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExchangeKeyBalance from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of futures
        if self.futures:
            _dict["futures"] = self.futures.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in spot (list)
        _items = []
        if self.spot:
            for _item_spot in self.spot:
                if _item_spot:
                    _items.append(_item_spot.to_dict())
            _dict["spot"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExchangeKeyBalance from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "api_key_id": obj.get("api_key_id"),
                "futures": (
                    FuturesBalance.from_dict(obj["futures"])
                    if obj.get("futures") is not None
                    else None
                ),
                "spot": (
                    [SpotBalance.from_dict(_item) for _item in obj["spot"]]
                    if obj.get("spot") is not None
                    else None
                ),
            }
        )
        return _obj
