# coding: utf-8

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class Coins(str, Enum):
    """
    Cryptocurrency identifiers for AI training data. Uses obfuscated identifiers for public API access.
    """

    """
    allowed enum values
    """
    ENUM_1 = "1"
    ENUM_2 = "2"
    ENUM_3 = "3"
    ENUM_4 = "4"
    ENUM_5 = "5"
    ENUM_6 = "6"
    ENUM_7 = "7"
    ENUM_8 = "8"
    ENUM_9 = "9"
    ENUM_10 = "10"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Coins from a JSON string"""
        return cls(json.loads(json_str))
