# coding: utf-8

"""
DEX AI API

API for DEX AI

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Self

from crypticorn.dex.client.models.risk import Risk
from crypticorn.dex.client.models.token_detail import TokenDetail


class TokenData(BaseModel):
    """
    Main model for complete token data
    """  # noqa: E501

    token: TokenDetail = Field(description="Token details")
    mcap: StrictStr = Field(description="Market cap")
    price: StrictStr = Field(description="Price")
    liquidity: StrictStr = Field(description="The liquidity of the token in USD")
    risk: Risk = Field(description="Risk assessment")
    buys: StrictInt = Field(description="Number of buys")
    sells: StrictInt = Field(description="Number of sells")
    txns: StrictInt = Field(description="Total transactions")
    holders: StrictInt = Field(description="Number of holders")
    __properties: ClassVar[List[str]] = [
        "token",
        "mcap",
        "price",
        "liquidity",
        "risk",
        "buys",
        "sells",
        "txns",
        "holders",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TokenData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of token
        if self.token:
            _dict["token"] = self.token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of risk
        if self.risk:
            _dict["risk"] = self.risk.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TokenData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "token": (
                    TokenDetail.from_dict(obj["token"])
                    if obj.get("token") is not None
                    else None
                ),
                "mcap": obj.get("mcap"),
                "price": obj.get("price"),
                "liquidity": obj.get("liquidity"),
                "risk": (
                    Risk.from_dict(obj["risk"]) if obj.get("risk") is not None else None
                ),
                "buys": obj.get("buys"),
                "sells": obj.get("sells"),
                "txns": obj.get("txns"),
                "holders": obj.get("holders"),
            }
        )
        return _obj
