# coding: utf-8

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class DataVersion(str, Enum):
    """
    Available data versions for AI model training datasets.
    """

    """
    allowed enum values
    """
    ENUM_1_DOT_0 = "1.0"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of DataVersion from a JSON string"""
        return cls(json.loads(json_str))
