# coding: utf-8

"""
Klines Service API

API for retrieving OHLCV data, funding rates, and symbol information from Binance.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class InternalExchange(str, Enum):
    """
    All exchanges we are using, including public (Exchange)
    """

    """
    allowed enum values
    """
    KUCOIN = "kucoin"
    BINGX = "bingx"
    BINANCE = "binance"
    BYBIT = "bybit"
    HYPERLIQUID = "hyperliquid"
    BITGET = "bitget"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of InternalExchange from a JSON string"""
        return cls(json.loads(json_str))
